import torch
from crtl.components.dual_encoder.pipeline.linker import Linker
from crtl.components.dual_encoder.pipeline.embedder import Embedder, EmbedderModule
from crtl.components.dual_encoder.pipeline.preprocessor import LKBPreprocessor
from crtl.components.dual_encoder.pipeline.train import PreproccessedDataSet
from crtl.datasets.medmentions.parse_medmentions import MedMentionCorpus

from transformers import AutoTokenizer, AutoModel

from crtl.lkb.lkb import LexicalKnowledgeBase
from crtl.s3_file_utils.s3_file_utils import cached_path

mps_device = torch.device("mps")

tokenizer = AutoTokenizer.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")

mention_embedder_module = EmbedderModule(
    AutoModel.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")
)
candidate_embedder_module = EmbedderModule(
    AutoModel.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")
)
medmention_preprocessor = MedmentionsPreprocessor(tokenizer, length=50, eval_split=0.1)

medmention_corpus = MedMentionCorpus.from_file(
    "/Users/coltonflowers/CRTL/src/crtl/datasets/medmentions/corpus_pubtator.txt"
)
preprocessed_training_dataset, preprocessed_val_dataset = medmention_preprocessor(
    medmention_corpus
)

lkb = LexicalKnowledgeBase.from_gzipped_jsonl(
    cached_path("s3://databrickstrenddata/crtl/lkb/UMLS_2022AB.gzip")
)

candidate_encodings = LKBPreprocessor(tokenizer).preprocess_lkb(lkb)

mention_embedder = Embedder(tokenizer, mention_embedder_module)
candidate_embedder = Embedder(tokenizer, candidate_embedder_module)
linker = Linker(mention_embedder, candidate_embedder, lkb, mps_device)

preprocessed_dataset = PreproccessedDataSet(
    preprocessed_training_dataset, preprocessed_val_dataset, candidate_encodings
)

linker.train(preprocessed_dataset)
