import tensordict
from torch import nn
from typing import Tuple

import torch
from crtl.components.dual_encoder.pipeline.embedder import Embedder, EmbedderModule


class LinkerModule(nn.Module):
    def __init__(
        self,
        mention_embedder_module: EmbedderModule,
        candidate_embedder_module: EmbedderModule,
    ):
        super().__init__()
        self.mention_embedder = mention_embedder_module
        self.candidate_embedder = candidate_embedder_module

    def forward(
        self, features: tensordict.TensorDict
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        mention_embeddings = self.mention_embedder.forward(
            features.get("mention_encodings")
        )
        # should run both of these together?
        negative_candidate_encodings: tensordict.TensorDict = features.get(
            "negative_candidate_encodings"
        )
        flattened_negative_candidate_encodings = negative_candidate_encodings.reshape(
            -1
        )
        positive_embeddings = self.candidate_embedder.forward(
            features.get("positive_candidate_encodings")
        )
        negative_embeddings = self.candidate_embedder.forward(
            flattened_negative_candidate_encodings
        ).view(
            size=(
                negative_candidate_encodings.shape[0],
                negative_candidate_encodings.shape[1],
                -1,
            )
        )

        return mention_embeddings, positive_embeddings, negative_embeddings

    @classmethod
    def from_embedders(cls, mention_embedder: Embedder, candidate_embedder: Embedder):
        return cls(mention_embedder.embedder_module, candidate_embedder.embedder_module)
